@@ -1,23 +1,30 @@
 # pylint: disable=unused-argument,invalid-name,line-too-long
 from __future__ import annotations
 
-from typing import List, Optional
+from typing import List
 
 from parse import parse
 from sqlalchemy import text as sql_text
 
+from alembic_utils.exceptions import SQLParseFailure
 from alembic_utils.replaceable_entity import ReplaceableEntity
 
 
 class PGFunction(ReplaceableEntity):
-    """ A PostgreSQL Function that can be versioned and replaced """
+    """A PostgreSQL Function compatible with `alembic revision --autogenerate`
+
+    **Parameters:**
+
+    * **schema** - *str*: A SQL schema name
+    * **signature** - *str*: A SQL function's call signature
+    * **definition** - *str*:  The remainig function body and identifiers
+
+    """
 
     @classmethod
-    def from_sql(cls, sql: str) -> Optional[PGFunction]:
-        """ Create an instance of PGFunction from a blob of sql """
-        template = (
-            "create{}function{:s}{schema}.{signature}{:s}returns{:s}{definition}"
-        )
+    def from_sql(cls, sql: str) -> PGFunction:
+        """Create an instance instance from a SQL string"""
+        template = "create{}function{:s}{schema}.{signature}{:s}returns{:s}{definition}"
         result = parse(template, sql.strip(), case_sensitive=False)
         if result is not None:
             return cls(
@@ -25,7 +32,7 @@ class PGFunction(ReplaceableEntity):
                 signature=result["signature"],
                 definition="returns " + result["definition"],
             )
-        return None
+        raise SQLParseFailure(f'Failed to parse SQL into PGFunction """{sql}"""')
 
     def to_sql_statement_create(self) -> str:
         """ Generates a SQL "create function" statement for PGFunction """
@@ -33,11 +40,23 @@ class PGFunction(ReplaceableEntity):
 
     def to_sql_statement_drop(self) -> str:
         """ Generates a SQL "drop function" statement for PGFunction """
-        return sql_text(f"DROP FUNCTION {self.schema}.{self.signature}")
+        template = "{function_name}({parameters})"
+        # Rmove default clause from drop statements because they don't parse
+        result = parse(template, self.signature, case_sensitive=False)
+        function_name = result["function_name"]
+        parameters_str = result["parameters"].strip()
+        # NOTE: Will fail if a text field has a default and that deafult contains a comma...
+        parameters: List[str] = parameters_str.split(",")
+        parameters = [x[: len(x.lower().split("default")[0])] for x in parameters]
+        parameters = [x.strip() for x in parameters]
+        drop_params = ", ".join(parameters)
+        return sql_text(f"DROP FUNCTION {self.schema}.{function_name}({drop_params})")
 
     def to_sql_statement_create_or_replace(self) -> str:
         """ Generates a SQL "create or replace function" statement for PGFunction """
-        return sql_text(f"CREATE OR REPLACE FUNCTION {self.schema}.{self.signature} {self.definition}")
+        return sql_text(
+            f"CREATE OR REPLACE FUNCTION {self.schema}.{self.signature} {self.definition}"
+        )
 
     @classmethod
     def from_database(cls, connection, schema="%") -> List[PGFunction]:
